@@ -1,7 +1,7 @@
 /*
    This file is part of ffmpeg-php
 
-   Copyright (C) 2004-2007 Todd Kirby (ffmpeg.php@gmail.com)
+   Copyright (C) 2004-2008 Todd Kirby (ffmpeg.php AT gmail.com)
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,55 +32,43 @@
 #ifndef FFMPEG_MOVIE_H
 #define FFMPEG_MOVIE_H
 
-#include <avcodec.h>
-#include <avformat.h>
-
 /* movie constructor */
-PHP_METHOD(ffmpeg_movie, __construct);
+FFMPEG_PHP_METHOD(ffmpeg_movie, __construct);
 
 /* movie methods */
-PHP_METHOD(ffmpeg_movie, getDuration);
-PHP_METHOD(ffmpeg_movie, getFrameCount);
-PHP_METHOD(ffmpeg_movie, getFrameRate);
-PHP_METHOD(ffmpeg_movie, getFileName);
-PHP_METHOD(ffmpeg_movie, getComment);
-PHP_METHOD(ffmpeg_movie, getTitle);
-PHP_METHOD(ffmpeg_movie, getAuthor);
-PHP_METHOD(ffmpeg_movie, getCopyright);
-PHP_METHOD(ffmpeg_movie, getAlbum);
-PHP_METHOD(ffmpeg_movie, getGenre);
-PHP_METHOD(ffmpeg_movie, getTrackNumber);
-PHP_METHOD(ffmpeg_movie, getYear);
-PHP_METHOD(ffmpeg_movie, getFrameWidth);
-PHP_METHOD(ffmpeg_movie, getFrameHeight);
-PHP_METHOD(ffmpeg_movie, getFrameNumber);
-PHP_METHOD(ffmpeg_movie, getPixelFormat);
-PHP_METHOD(ffmpeg_movie, getBitRate);
-PHP_METHOD(ffmpeg_movie, hasAudio);
-PHP_METHOD(ffmpeg_movie, hasVideo);
-PHP_METHOD(ffmpeg_movie, getNextKeyFrame);
-PHP_METHOD(ffmpeg_movie, getFrame);
-PHP_METHOD(ffmpeg_movie, getVideoCodec);
-PHP_METHOD(ffmpeg_movie, getAudioCodec);
-PHP_METHOD(ffmpeg_movie, getAudioChannels);
-PHP_METHOD(ffmpeg_movie, getAudioSampleRate);
-PHP_METHOD(ffmpeg_movie, getAudioBitRate);
-PHP_METHOD(ffmpeg_movie, getVideoBitRate);
-PHP_METHOD(ffmpeg_movie, getPixelAspectRatio);
-PHP_METHOD(ffmpeg_movie, getVideoStreamId);
-PHP_METHOD(ffmpeg_movie, getAudioStreamId);
-
-typedef struct {
-    AVFormatContext *fmt_ctx;
-    AVCodecContext *codec_ctx[MAX_STREAMS];
-    int64_t last_pts;
-    int frame_number;
-    long rsrc_id;
-} ff_movie_context;
+FFMPEG_PHP_METHOD(ffmpeg_movie, getDuration);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getFrameCount);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getFrameRate);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getFileName);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getComment);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getTitle);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getAuthor);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getCopyright);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getAlbum);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getGenre);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getTrackNumber);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getYear);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getFrameWidth);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getFrameHeight);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getFrameNumber);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getPixelFormat);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getBitRate);
+FFMPEG_PHP_METHOD(ffmpeg_movie, hasAudio);
+FFMPEG_PHP_METHOD(ffmpeg_movie, hasVideo);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getNextKeyFrame);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getFrame);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getVideoCodec);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getAudioCodec);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getAudioChannels);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getAudioSampleRate);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getAudioBitRate);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getVideoBitRate);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getPixelAspectRatio);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getVideoStreamId);
+FFMPEG_PHP_METHOD(ffmpeg_movie, getAudioStreamId);
 
 #endif // FFMPEG_MOVIE_H
 
-
 /*
  * Local variables:
  * tab-width: 4
